"""
List of valid field options that can be used for parsing tweets.
"""

FIELD_OPTIONS = [
    "archived_urlkey",
    "archived_timestamp",
    "parsed_archived_timestamp",
    "archived_tweet_url",
    "parsed_archived_tweet_url",
    "original_tweet_url",
    "parsed_tweet_url",
    "available_tweet_text",
    "available_tweet_is_RT",
    "available_tweet_info",
    "archived_mimetype",
    "archived_statuscode",
    "archived_digest",
    "archived_length",
]
